const fs = require('fs');
const path = require('path');
const walker = require('walker');
const del = require('del');
const co = require('co');
const inquirer = require('inquirer');
const chalk = require('chalk');

const lib = require('./lib/');
const database = require('./lib/database');
const fileTypes = require('./lib/fileTypes');
const _ = require('lodash');

/**
 * Inquirer which database to use.
 */
const inquirerQ = function () {
  const questions = [
    {
      type: 'input',
      name: 'dbName',
      message: `Please input name of the database.`,
      default: 'mysql',
      validate: value => /\w+/.test(value)
    },
    {
      type: 'input',
      name: 'distRoot',
      message: 'Which folder do you want to save to.',
      default: process.env.DIST_ROOT || 'dist',
    }
  ]
  return inquirer.prompt(questions)
}

/**
 * 生成 view 文件。 
 * @param {String} file 
 * @param {String} dist 
 * @param {String} database name 
 * @param {Array} tables 
 */
const generateViews = function (file, dist, dbName, tables, templateRoot) {
  if (!fs.existsSync(file)) {
    return new Error('文件不存在')
  }

  const template = lib.getTemplateF(file);
    
  for (let table of tables) {
    table.comment = table.comment || table.name;
    const realName = table.comment + path.extname(fileTypes.view);
    const _path = lib.getForLoopTDP(file, realName, dist, templateRoot);

    let context = {}
    context._ = _;
    co(database.showColumns(dbName, table.name))
      .then(fields => {
        context.model = table.comment;
        context.fields = fields;
        const content = template(context);
        lib.save2File(_path, content);
      })
      .catch(err => {
        console.error(err);
        process.exit(0);
      })
  }
}

/**
 * 生成 routes 文件
 * @param {String} file 
 * @param {String} dist 
 * @param {Array} tables 
 */
const generateRoutes = function (file, dist, tables, templateRoot) {
  if (!fs.existsSync(file)) {
    return new Error('文件不存在')
  }

  const template = lib.getTemplateF(file);

  let context = {}
  context._ = _;
  context.tables = tables;
  const content = template(context);
  const _path = lib.getForLoopTDP(file, 'index.js', dist, templateRoot);
  lib.save2File(_path, content);
}

/**
 * 利用默认上下文，生成自身编译文件
 * @param {String} file 
 * @param {String} dist 
 * @param {Array} tables 
 */
const generateSelf = function (file, dist, tables, templateRoot) {
  if (!fs.existsSync(file)) {
    return new Error('文件不存在')
  }

  const template = lib.getTemplateF(file);

  let context = {}
  context._ = _;
  context.tables = tables;
  const content = template(context);
  const _path = lib.getTemplateDistP(file, dist, templateRoot);
  lib.save2File(_path, content);
}

const scaffold = function* () {
  require('dotenv').config({
    path: process.env.ENV_FILE || '.env'
  });

  const { dbName, distRoot } = yield inquirerQ();
  const dist = path.join(process.cwd(), distRoot);
  const tables = yield database.showTables(dbName);

  // Template settings.
  const _templateRoot = process.env.TEMPLATE_ROOT || 'templates';
  const templateRoot = path.isAbsolute(_templateRoot)
    ? _templateRoot : path.join(__dirname, _templateRoot);
  const templateExt = process.env.TEMPLATE_EXTENSION || '.ejs';

  del.sync([dist]);

  walker(templateRoot)
    .filterDir(function (dir, stat) {
      
      const _path = path.join(templateRoot, 'node_modules')
      if (dir === _path) {
        return false;
      }
      return true;
    })
    .on('file', function (file, stat) {
      const basename = path.basename(file);
      const isTpl = basename.endsWith(templateExt);

      if (isTpl) {
        const isForloopT = basename.startsWith('_');
        if (isForloopT) {
          // 如果模板类型是 view
          if (fileTypes.view === path.parse(basename).name) {

            generateViews(file, dist, dbName, tables, templateRoot);
          } else if (fileTypes.routes === path.parse(basename).name) {
            // 如果模板类型是 routes

            generateRoutes(file, dist, tables, templateRoot);
          }
        } else {
          // 保存自身的普通模板，默认的 context 为 tables
          generateSelf(file, dist, tables, templateRoot);
        }
      } else {
        // 如果不是模板，则复制该文件到相应路径
        lib.copyStatic(file, dist, templateRoot);
      }
    });
}

module.exports = scaffold;
